#!/bin/bash
# 更改日期:2025/01/02



#==================== 重要變數區 ====================

#----------------------
# 設定 /backup 路徑和檢查文件
# 重要!!!!! 請一定要設置 /etc/rc.d/rc.local 
# 請使用 blkid 指令確認 新磁碟 uuid 
# 範例如下
# iscsiadm -m node -T [iqn名稱] --login ; mount -U [iscsi磁碟UUID] [/backup or /backup2]
# iscsiadm -m node -T iqn.cp31-zvol --login ; mount -U 19edcd89-43a6-493f-a979-184948b639d0 /backup


## /etc/csf/csf.allow 裡面要加以下格式
## tcp|out|d=3260|d=NAS的IP # NAS-編號 iSCSI /backup
## tcp|out|d=3260|d=10.100.14.67 # NAS-N9 iSCSI /backup

### 如果有第二顆 iSCSI 磁區需要監控，請將下方 MOUNT_POINT 修改 => (需將此script 另存新路徑+設置排程)。
### /etc/csf/csf.allow 裡面要加以下格式
### tcp|out|d=3260|d=NAS的IP # NAS-編號 iSCSI /backup2
### tcp|out|d=3260|d=10.100.14.67 # NAS-N9 iSCSI /backup2

#Jetbackup 路徑
MOUNT_POINT="/backup"

#自動抓取 /etc/csf/csf.allow 裡面設置 iSCSI 允許的 NAS_IP與備份碟磁區.
NAS_IP=$(cat /etc/csf/csf.allow |grep "NAS.*iSCSI" |grep "$MOUNT_POINT"$ | sed -n 's/.*d=\([0-9]\{1,3\}\.[0-9]\{1,3\}\.[0-9]\{1,3\}\.[0-9]\{1,3\}\).*/\1/p')

#----------------------


# 此變數會依據上方設定而變化，故無需修改!
CHECK_FILE="$MOUNT_POINT/iscsi_check"

#================================================




#此主機的hostname
host=`hostname -s | tr [:lower:] [:upper:]`

# 從 /etc/rc.d/rc.local 提取 iSCSI 設定
hdd_file_iqn=$(grep -E "iscsiadm.*file.*${MOUNT_POINT}$" /etc/rc.d/rc.local | awk -F'-T ' '{print $2}' | awk '{print $1}')
hdd_zvol_iqn=$(grep -E "iscsiadm.*zvol.*${MOUNT_POINT}$" /etc/rc.d/rc.local | awk -F'-T ' '{print $2}' | awk '{print $1}')
hdd_file_uuid=$(grep -E "iscsiadm.*file.*${MOUNT_POINT}$" /etc/rc.d/rc.local | awk -F'-U ' '{print $2}' | awk '{print $1}')
hdd_zvol_uuid=$(grep -E "iscsiadm.*zvol.*${MOUNT_POINT}$" /etc/rc.d/rc.local | awk -F'-U ' '{print $2}' | awk '{print $1}')


#今天日期(20xx-xx-xx)
today_date=$(date +%Y%m%d)
#今天是星期幾
today_of_week=$(date +%A)
#今天是幾號
today_day=$(date +%d)


#時間戳 20XX-XX-XX XX:XX:XX
backuptime=$(date +"%F %T")



## 本機 log 存放路徑，超過 14 天 log 會自動被刪除
logpath="/root/iscsi_check"
DATE_14_AGO=`date --date='14 days ago' +"%Y%m%d"`
/bin/rm -f $logpath/$DATE_14_AGO.iscsi_check.log 2> /dev/null


#今天備份log 路徑
LOG_FILE="${logpath}/${today_date}.iscsi_check.log"

mkdir -p $logpath

# 記錄訊息並加上時間戳，根據選項控制輸出模式
log_message() {
    local mode=$1
    local message=$2
    local timestamp=$(date "+%Y-%m-%d %H:%M:%S")
    local log_entry="$timestamp: $message"

    case $mode in
        only_echo)
            # 僅顯示訊息到終端
            echo "$log_entry"
            ;;
        echo_log)
            # 顯示訊息到終端並寫入 log 檔案
            echo "$log_entry" | tee -a "$LOG_FILE"
            ;;
        only_log)
            # 僅寫入 log 檔案
            echo "$log_entry" >> "$LOG_FILE"
            ;;
        *)
            echo "無效的模式: $mode"
            ;;
    esac
}


# 刷新訊息
refresh_message() {

    target_iqn=""
    target_uuid=""

    # 判斷 hdd_file_iqn 和 hdd_zvol_iqn 是否存在
    if [[ -n "$hdd_file_iqn" ]]; then
        target_iqn=$hdd_file_iqn
    elif [[ -n "$hdd_zvol_iqn" ]]; then
        target_iqn=$hdd_zvol_iqn
    else
        :
    fi

    # 判斷 hdd_file_uuid 和 hdd_zvol_uuid 是否存在
    if [[ -n "$hdd_file_uuid" ]]; then
        target_uuid=$hdd_file_uuid
        # 您可以在這裡添加對 hdd_file_uuid 的處理邏輯
    elif [[ -n "$hdd_zvol_uuid" ]]; then
        target_uuid=$hdd_zvol_uuid
    else
        :
    fi



    # 定義凌晨12點前的訊息
    messages_midnight="
$host $MOUNT_POINT 目前有 iSCSI Read-only file system 異常，目前已自動重新掛載。

請確認 Jetbackup 備份排程是否有正常執行。
"
    
    # 定義正常時間段的訊息
    messages_normal=" 
$host $MOUNT_POINT 目前有 iSCSI Read-only file system 異常，目前已自動重新掛載。
"

    # 定義iSCSI重新掛載異常的詳細處理步驟
    messages_failed=" 
$host $MOUNT_POINT iSCSI 重新掛載異常，請依照以下流程檢查:

1. 於 $MOUNT_POINT 試著 touch 檔案確認。

2. 如無法建立檔案，則 輸入以下指令:

    umount $MOUNT_POINT -l
    iscsiadm -m node -T $target_iqn --logout
    iscsiadm -m node -T $target_iqn --login
    mount -U $target_uuid $MOUNT_POINT


    接著使用以下指令確認是否有正常掛載:
    mountpoint "$MOUNT_POINT"

3. 如發生時間是凌晨及早上，請確認 Jetbackup 備份排程是否有正常執行。

4. 如以上操作後仍無法掛載成功，請通知二線或主管協助確認問題。
"

    # 判斷當前時間是否在凌晨至中午12點之間
    if [[ "$current_time" > "00:00:00" && "$current_time" < "12:00:00" ]]; then
        messages="$messages_midnight"
    else
        messages="$messages_normal"
    fi
}


# 登入 iSCSI 來源
login_iscsi_source() {
    log_message "echo_log" "開始搜尋 iSCSI 目標位於 $NAS_IP..."	
    iscsiadm -m discovery -t sendtargets -p $NAS_IP 
    local iqn="$1"
    iscsiadm -m node -T "$iqn" --login
    sleep 2  # 等待一會兒讓掛載完成
}


# 掛載 iSCSI 磁碟
mount_iscsi_disk() {
    local disk_uuid="$1"
    log_message "echo_log" "正在掛載磁碟 UUID: $disk_uuid 到 $MOUNT_POINT"
    mount -U "$disk_uuid" "$MOUNT_POINT"
    sleep 2  # 等待一會兒讓掛載完成
}

# 卸載掛載點並登出 iSCSI
cleanup_mount_and_logout() {
    local iqn="$1"
    log_message "echo_log" "正在清理掛載點並登出 iSCSI 來源: $iqn"
    umount "$MOUNT_POINT" -l 2>/dev/null
    iscsiadm -m node -T "$iqn" --logout
}

# 檢查 iSCSI 掛載點的狀態
check_iscsi_mount_status() {
    if mountpoint -q "$MOUNT_POINT"; then
        return 0  # 是正確的 iSCSI 掛載點
    else
        return 1  # 不是掛載點
    fi
}

check_writeable() {
    check_iscsi_mount_status  # 檢查掛載狀態
    local mount_status=$?  # 保存掛載狀態碼

    if [ $mount_status -eq 0 ]; then
        log_message "echo_log" "檢查掛載點: $MOUNT_POINT, 掛載成功"
        # 嘗試寫入檔案，檢查是否可寫
        local test_file="$CHECK_FILE"
        if touch $test_file >/dev/null 2>&1; then
            log_message "echo_log" "成功建立測試檔案 $test_file"
            if rm -f $test_file >/dev/null 2>&1; then
                log_message "echo_log" "成功刪除測試檔案 $test_file"
                log_message "echo_log" "$MOUNT_POINT 可寫"
                return 0  # 可寫
            else
                log_message "echo_log" "刪除測試檔案 $test_file 失敗"
                return 1  # 無法刪除
            fi
        else
            log_message "echo_log" "建立測試檔案 $test_file 失敗 (檔案系統可能為唯讀)"
            return 1  # 無法建立檔案
        fi
    else
        log_message "echo_log" "$MOUNT_POINT 無法掛載"
        return 2  # 無法掛載
    fi
}

# 定義執行掛載操作的函式
perform_iscsi_mount() {
    log_message "echo_log" "執行 iSCSI 掛載作業..."

    # 如果已掛載，先清理掛載
    if check_iscsi_mount_status; then
        log_message "echo_log" "檢測到 $MOUNT_POINT 已掛載，正在執行清理..."
        if [ -n "$hdd_file_iqn" ]; then
            cleanup_mount_and_logout "$hdd_file_iqn" || { 
                log_message "echo_log" "清理 $hdd_file_iqn 失敗"; 
                return 1; 
            }
        elif [ -n "$hdd_zvol_iqn" ]; then
            cleanup_mount_and_logout "$hdd_zvol_iqn" || { 
                log_message "echo_log" "清理 $hdd_zvol_iqn 失敗"; 
                return 1; 
            }
        else
            log_message "echo_log" "找不到需要清理的 iSCSI 資源"
            return 1
        fi
    fi

    # 根據資源類型執行掛載
    if [ -n "$hdd_file_iqn" ]; then
        log_message "echo_log" "使用 file 類型掛載 $hdd_file_iqn"
        login_iscsi_source "$hdd_file_iqn" || { 
            log_message "echo_log" "登入 iSCSI 來源 $hdd_file_iqn 失敗"; 
            return 1; 
        }
        mount_iscsi_disk "$hdd_file_uuid" || { 
            log_message "echo_log" "掛載磁碟 $hdd_file_uuid 失敗"; 
            return 1; 
        }
    elif [ -n "$hdd_zvol_iqn" ]; then
        log_message "echo_log" "使用 zvol 類型掛載 $hdd_zvol_iqn"
        login_iscsi_source "$hdd_zvol_iqn" || { 
            log_message "echo_log" "登入 iSCSI 來源 $hdd_zvol_iqn 失敗"; 
            return 1; 
        }
        mount_iscsi_disk "$hdd_zvol_uuid" || { 
            log_message "echo_log" "掛載磁碟 $hdd_zvol_uuid 失敗"; 
            return 1; 
        }
    else
        log_message "echo_log" "無法找到適當的 iSCSI 資源進行掛載"
        return 1  # 掛載失敗
    fi

    # 確認掛載是否成功
    if check_iscsi_mount_status; then
        log_message "echo_log" "$MOUNT_POINT 掛載成功"
        return 0
    else
        log_message "echo_log" "$MOUNT_POINT 掛載失敗"
        return 1
    fi
}

##### 主程式

echo "===============================================" | tee -a "$LOG_FILE"


# 呼叫函式並捕獲返回值
check_writeable
writeable_status=$?  # 抓取返回的狀態碼

# 確保 $writeable_status 是有效整數
if [[ "$writeable_status" =~ ^[0-2]$ ]]; then
    if [ "$writeable_status" -ne 0 ]; then
        log_message "echo_log" "檢測到 $MOUNT_POINT 異常，嘗試開始重新掛載作業..."

        # 嘗試掛載 iSCSI 資源
        if perform_iscsi_mount; then
            log_message "echo_log" "掛載完成，進行二次檢查..."

	    check_writeable
	    writeable_status=$?  # 抓取返回的狀態碼

	    refresh_message


            case "$writeable_status" in
                0)
		    log_message "echo_log" "掛載成功並可正常寫入"
                    echo $messages | mutt -s "$host iSCSI 自動掛載成功通知" support@twnoc.net,aaaacrwo32zcdpoxsm5jdeyruy@twnoc.slack.com
                    ;;
                1)
		    log_message "echo_log" "警告：掛載成功，但不可寫入"
                    echo "警告：掛載成功，但不可寫入" | mutt -s "$host iSCSI 自動掛載警告" support@twnoc.net,aaaacrwo32zcdpoxsm5jdeyruy@twnoc.slack.com
                    ;;
                2)
		    log_message "echo_log" "錯誤：掛載失敗，$MOUNT_POINT 未正確掛載"
                    echo $messages_failed | mutt -s "$host iSCSI 自動掛載失敗警告" support@twnoc.net,aaaacrwo32zcdpoxsm5jdeyruy@twnoc.slack.com
                    ;;
                *)
		    log_message "echo_log" "未知狀態：$writeable_status，請檢查腳本和系統狀態"
                    echo "發生未知錯誤，請通知二線協助確認" | mutt -s "$host iSCSI 自動掛載未知警告" support@twnoc.net,aaaacrwo32zcdpoxsm5jdeyruy@twnoc.slack.com
                    ;;
            esac
        else
	    log_message "echo_log" "掛載失敗，無法執行掛載操作"
            echo "掛載失敗，無法執行掛載操作，請通知二線協助確認" | mutt -s "$host iSCSI 自動掛載失敗警告" support@twnoc.net,aaaacrwo32zcdpoxsm5jdeyruy@twnoc.slack.com
        fi
    else
	log_message "echo_log" "檢查完成，$MOUNT_POINT 狀態正常，無需重新掛載。"
    fi
else
    log_message "echo_log" "錯誤：無效的 writeable_status 值，請檢查"
    echo "script 未知錯誤，請通知二線協助確認" | mutt -s "$host iSCSI 自動掛載錯誤" support@twnoc.net,aaaacrwo32zcdpoxsm5jdeyruy@twnoc.slack.com
fi

echo "===============================================" | tee -a "$LOG_FILE"
echo "" | tee -a "$LOG_FILE"
echo "" | tee -a "$LOG_FILE"
